/* PrettyOS is the result of a stepwise OS development
 * shown at www.henkessoft.de */

#ifndef OS_H
#define OS_H

#define TRUE		1
#define FALSE		0
typedef unsigned int   size_t;
typedef unsigned int   UINT;
typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;
typedef signed char    CHAR;

#define ASSERT(b) ((b) ? (void)0 : panic_assert(__FILE__, __LINE__, #b))
extern void panic_assert(const char* file, ULONG line, const char* desc);

#define KQSIZE    100 // size of key queue





/* This defines the operatings system common data area */
typedef struct oda
{
    ULONG COM1, COM2, COM3, COM4; // address
    ULONG LPT1, LPT2, LPT3, LPT4; // address

    ULONG Memory_Size;            // Memory size in Byte

    // Key Flags
    UCHAR left_shift_key_pressed;
    UCHAR right_shift_key_pressed;

    UCHAR left_ctrl_key_pressed;
    UCHAR right_ctrl_key_pressed;

    UCHAR left_ALT_key_pressed;
    UCHAR right_ALT_key_pressed; // Alt Gr

    UCHAR caps_lock_active;
    UCHAR num_lock_active;
    UCHAR scroll_lock_active;

    UCHAR left_win_key_pressed;
    UCHAR right_win_key_pressed;

    UCHAR function_key_pressed[12];

    // Key Queue
    UCHAR  KEYQUEUE[KQSIZE];   // circular queue buffer
    UCHAR* pHeadKQ;            // pointer to the head of valid data
    UCHAR* pTailKQ;            // pointer to the tail of valid data
}oda_t;

/* This defines what the stack looks like after an ISR was running */
typedef struct regs
{
    ULONG gs, fs, es, ds;
    ULONG edi, esi, ebp, esp, ebx, edx, ecx, eax;
    ULONG int_no, err_code;
    ULONG eip, cs, eflags, useresp, ss;
}registers_t;

extern void k_clear_screen();
extern void settextcolor(UCHAR forecolor, UCHAR backcolor);
extern void putch(UCHAR c);
extern void puts(UCHAR* text);
extern void scroll();
extern void k_printf(UCHAR* message, UINT line, UCHAR attribute);
extern void set_cursor(UCHAR x, UCHAR y);
extern void update_cursor();
extern void move_cursor_right();
extern void move_cursor_left();
extern void move_cursor_home();
extern void move_cursor_end();

extern void save_cursor();
extern void restore_cursor();
extern void printformat (char *args, ...);

extern void timer_handler(struct regs* r);
extern void timer_wait (ULONG ticks);
extern void sleepSeconds (ULONG seconds);
extern void systemTimer_setFrequency( ULONG freq );
extern void timer_install();
extern void timer_uninstall();

extern void keyboard_init();
extern UCHAR k_getch();
extern UCHAR FetchAndAnalyzeScancode();
extern void keyboard_handler(struct regs* r);

extern void outportb(UINT port, UINT val);
inline UINT inportb(UINT port);
extern void* k_memset(void* dest, char val, size_t count);
extern USHORT* k_memsetw(USHORT* dest, USHORT val, size_t count);
extern void* k_memcpy(void* dest, const void* src, size_t count);
extern size_t k_strlen(const char* str);
extern void reboot();

extern void cli();
extern void sti();
extern void k_itoa(int value, char* valuestring);
extern void k_i2hex(UINT val, UCHAR* dest, int len);
extern void float2string(float value, int decimal, char* valuestring);

extern void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran);
extern void gdt_install();

extern void idt_set_gate(unsigned char num, unsigned long base, unsigned short sel, unsigned char flags);
extern void idt_install();

extern void isrs_install();
extern void irq_install();
extern void timer_install();
extern void keyboard_install();
extern void fault_handler(struct regs* r);
extern void irq_install_handler(int irq, void (*handler)(struct regs* r));
extern void irq_uninstall_handler(int irq);
extern void irq_remap(void);
extern void irq_handler(struct regs* r);

extern int k_abs(int i);
extern int k_power(int base,int n);

/*
extern void beep();
extern void sound(UINT frequency);
extern void noSound();
*/

extern void paging_install();

typedef void* type_t;
typedef CHAR (*lessthan_predicate_t)(type_t,type_t);


typedef struct
{
    type_t* array;
    ULONG size;
    ULONG max_size;
    lessthan_predicate_t less_than;
} ordered_array_t;

typedef struct
{
    ordered_array_t index;
    ULONG start_address; // The start of our allocated space.
    ULONG end_address;   // The end of our allocated space. May be expanded up to max_address.
    ULONG max_address;   // The maximum address the heap can be expanded to.
    UCHAR supervisor;    // Should extra pages requested by us be mapped as supervisor-only?
    UCHAR readonly;      // Should extra pages requested by us be mapped as read-only?
} heap_t;




#endif
